﻿//-----------------------------------------------------------------------------
// ACI PIM CPN Sensor sample program
// Cpn sensor device sample console application
//-----------------------------------------------------------------------------
// 2020 Analytical Control Instruments GmbH
//-----------------------------------------------------------------------------


using System;
using System.Globalization;


namespace Cpn.Sensor
{
  class Program
  {
    static void Main ( string[] args )
    {
      //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      var portname = "COM1";
      if ( 1 == args.Length ) portname = args[0];

      Console.WriteLine ( "Calibrating on port " + portname );
      Console.WriteLine ();

      //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      try
      {
        if ( !Device.Identify ( portname ) )
          throw new Exception ( "Device identification FAILED on " + portname );

        using ( var device = new Device ( portname ) )
        {
          device.Connect ();

          // get current calibration values
          var curcalib = CalibValues.FromMessageValues(device.Transmit("pid.calib ?").ParameterAsDoubleList);

          Console.WriteLine ( "Module current calibration values :" );
          Console.WriteLine ();
          curcalib.WriteValuesToConsole ();

          //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
          var valspangasconc = 100.0;
          Console.WriteLine ();
          Console.WriteLine ( "Enter SPAN gas point concentration value [ppm]. Format is 100.0" );
          Console.Write ( "Calibration gas : " );
          var svalspangasconc = Console.ReadLine();

          if ( !double.TryParse ( svalspangasconc, NumberStyles.Float, NumberFormatInfo.InvariantInfo, out valspangasconc ) )
          {
            Console.WriteLine ( "Invalid number format. Format must be 100.0. Press key to exit." );
            Console.ReadKey ();
            return;
          }

          var newcalib = new CalibValues();
          newcalib.ConcentrationZero = 0.0;
          newcalib.ConcentrationSpan = valspangasconc;

          //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
          Console.WriteLine (); Console.WriteLine ();
          Console.Write ( "Apply fresh air to device. Wait for signal is stabilized (at least 60 sec)." );
          Console.WriteLine ( "then press a key." + Environment.NewLine );

          device.Transmit ( "pid.start" );

          while ( !Console.KeyAvailable )
          {
            var values = device.PidValues();
            newcalib.CurrentZero = values[1];

            Console.Write ( "  {0,6:F1} ppm Isobutene sensor compensated current is {1,8:F2} pA\r", newcalib.ConcentrationZero, newcalib.CurrentZero );
          }

          Console.ReadKey ();

          //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
          Console.WriteLine (); Console.WriteLine ();
          Console.Write ( "Apply calibration gas to device. Wait for signal is stabilized (at least 120 sec)." );
          Console.WriteLine ( "then press a key." + Environment.NewLine );

          while ( !Console.KeyAvailable )
          {
            var values = device.PidValues();
            newcalib.CurrentSpan = values[1];

            Console.Write ( "  {0,6:F1} ppm Isobutene sensor compensated current is {1,8:F2} pA\r", newcalib.ConcentrationSpan, newcalib.CurrentSpan );
          }

          Console.ReadKey ();
          Console.WriteLine (); Console.WriteLine ();

          //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
          Console.WriteLine ( "Module new calibration values :" );
          Console.WriteLine ();
          newcalib.WriteValuesToConsole ();

          Console.WriteLine ();
          Console.WriteLine ( "Writing calibration data to device ..." );
          var txmsg = new Communication.Message("pid.calib");
          txmsg.ParameterAsDoubleList = newcalib.ToMessageValues ();

          var rxmsg = device.Transmit ( txmsg );
          
          // here with no error message the calibration is successful and can be saved
          device.Transmit ( "pid.savedata" );

          //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
          device.PidStop ();

          Console.WriteLine ( "Device is calibrated. Press a key to exit." );
          Console.ReadKey ();
          //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
        } // using 
      }
      catch ( Exception exc )
      {
        Console.ForegroundColor = ConsoleColor.Red;
        Console.WriteLine ();
        Console.WriteLine ( "ERROR : " );
        Console.WriteLine ( exc.Message );
        Console.WriteLine ();
        Console.ForegroundColor = ConsoleColor.White;
        Console.WriteLine ( "Press any key to exit" );
        Console.ReadKey ();
      }
      //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    }
  }
}
